@@ -1,7 +1,9 @@
 /*--------------------------------*-C-*---------------------------------*
  * File:	menubar.c
  *----------------------------------------------------------------------*
- * Copyright (C) 1997 1998  mj olesen <olesen@me.QueensU.CA>
+ * $Id: menubar.c,v 1.24 1998/11/16 17:26:44 mason Exp $
+ *
+ * Copyright (C) 1997,1998  mj olesen <olesen@me.QueensU.CA>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,85 +18,53 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
  *----------------------------------------------------------------------*
  * refer.html (or refer.txt) contains up-to-date documentation.  The
  * summary that appears at the end of this file was taken from there.
  *----------------------------------------------------------------------*/
 
 #include "rxvt.h"		/* NECESSARY */
+#include "version.h"
+#include "menubar.h"
 
-/* #define DEBUG_MENU */
-/* #define DEBUG_MENU_LAYOUT */
-/* #define DEBUG_MENUBAR_STACKING */
-
-#if (MENUBAR_MAX)
-#define HSPACE		1	/* one space */
-#define MENU_MARGIN	2
-#define menu_height()	(TermWin.fheight + 2 * MENU_MARGIN)
-
-#define MENU_DELAY_USEC	250000	/* 1/4 sec */
-
-#define SEPARATOR_HALFHEIGHT	(SHADOW + 1)
-#define SEPARATOR_HEIGHT	(2 * SEPARATOR_HALFHEIGHT)
-#define isSeparator(name)	((name)[0] == '\0')
-
-#define SEPARATOR_NAME		"-"
-#define MENUITEM_BEG		'{'
-#define MENUITEM_END		'}'
-#define COMMENT_CHAR		'#'
-
-#define DOT	"."
-#define DOTS	".."
-
-#define Menu_PixelWidth(menu)	(2 * SHADOW + \
-Width2Pixel ((menu)->width + 3 * HSPACE))
+#ifdef MENUBAR
+#define Menu_PixelWidth(menu)					\
+    (2 * SHADOW + Width2Pixel ((menu)->width + 3 * HSPACE))
 
 static GC       topShadowGC, botShadowGC, neutralGC, menubarGC;
 
-struct menu_t;
-
 static int      menu_readonly = 1;	/* okay to alter menu? */
 static int      Arrows_x = 0;
 static const struct {
     char            name;	/* (l)eft, (u)p, (d)own, (r)ight */
-    unsigned char  *str;	/* str[0] = strlen (str+1) */
+    unsigned char   str[4];	/* str[0] = strlen (str+1) */
 } Arrows[NARROWS] = {
-
-    {
-	'l', "\003\033[D"
-    },
-    {
-	'u', "\003\033[A"
-    },
-    {
-	'd', "\003\033[B"
-    },
-    {
-	'r', "\003\033[C"
-    }
+    { 'l', "\003\033[D" },
+    { 'u', "\003\033[A" },
+    { 'd', "\003\033[B" },
+    { 'r', "\003\033[C" }
 };
 
 #if (MENUBAR_MAX > 1)
 static int      Nbars = 0;
 static bar_t   *CurrentBar = NULL;
-
 #else				/* (MENUBAR_MAX > 1) */
 static bar_t    BarList;
 static bar_t   *CurrentBar = &BarList;
-
 #endif				/* (MENUBAR_MAX > 1) */
 
 static menu_t  *ActiveMenu = NULL;	/* currently active menu */
+#endif
 
 /*}}} */
 
+#ifdef MENUBAR
 /*
  * find an item called NAME in MENU
  */
-/* PROTO */
+/* INTPROTO */
 menuitem_t     *
-menuitem_find(menu_t * menu, char *name)
+menuitem_find(const menu_t *menu, const char *name)
 {
     menuitem_t     *item;
 
@@ -112,13 +82,15 @@ menuitem_find(menu_t * menu, char *name)
     }
     return item;
 }
+#endif
 
+#ifdef MENUBAR
 /*
  * unlink ITEM from its MENU and free its memory
  */
-/* PROTO */
+/* INTPROTO */
 void
-menuitem_free(menu_t * menu, menuitem_t * item)
+menuitem_free(menu_t *menu, menuitem_t *item)
 {
 /* disconnect */
     menuitem_t     *prev, *next;
@@ -153,22 +125,24 @@ menuitem_free(menu_t * menu, menuitem_t * item)
 	FREE(item->name2);
     FREE(item);
 }
+#endif
 
+#ifdef MENUBAR
 /*
  * sort command vs. terminal actions and
  * remove the first character of STR if it's '\0'
  */
-/* PROTO */
+/* INTPROTO */
 int
-action_type(action_t * action, unsigned char *str)
+action_type(action_t *action, unsigned char *str)
 {
     unsigned int    len;
 
 #if defined (DEBUG_MENU) || defined (DEBUG_MENUARROWS)
     len = strlen(str);
-    printf("(len %d) = %s\n", len, str);
+    fprintf(stderr, "(len %d) = %s\n", len, str);
 #else
-    len = Str_escaped(str);
+    len = Str_escaped((char *)str);
 #endif
 
     if (!len)
@@ -194,10 +168,12 @@ action_type(action_t * action, unsigned char *str)
 
     return 0;
 }
+#endif
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 int
-action_dispatch(action_t * action)
+action_dispatch(action_t *action)
 {
     switch (action->type) {
     case MenuTerminalAction:
@@ -214,9 +190,11 @@ action_dispatch(action_t * action)
     }
     return 0;
 }
+#endif
 
+#ifdef MENUBAR
 /* return the arrow index corresponding to NAME */
-/* PROTO */
+/* INTPROTO */
 int
 menuarrow_find(char name)
 {
@@ -227,9 +205,11 @@ menuarrow_find(char name)
 	    return i;
     return -1;
 }
+#endif
 
+#ifdef MENUBAR
 /* free the memory associated with arrow NAME of the current menubar */
-/* PROTO */
+/* INTPROTO */
 void
 menuarrow_free(char name)
 {
@@ -255,8 +235,10 @@ menuarrow_free(char name)
 	    menuarrow_free(Arrows[i].name);
     }
 }
+#endif
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 void
 menuarrow_add(char *string)
 {
@@ -273,12 +255,12 @@ menuarrow_add(char *string)
 	NULL, 0
     }, *cur, parse[NARROWS];
 
-    rmemset(parse, 0, sizeof(parse));
+    MEMSET(parse, 0, sizeof(parse));
 
-/* printf ("add arrows = `%s'\n", string); */
+/* fprintf(stderr, "add arrows = `%s'\n", string); */
     for (p = string; p != NULL && *p; string = p) {
 	p = (string + 3);
-    /* printf ("parsing at %s\n", string); */
+    /* fprintf(stderr, "parsing at %s\n", string); */
 	switch (string[1]) {
 	case 'b':
 	    cur = &beg;
@@ -327,19 +309,19 @@ menuarrow_add(char *string)
 
 #ifdef DEBUG_MENUARROWS
     cur = &beg;
-    printf("<b>(len %d) = %.*s\n",
-	   cur->len, cur->len, (cur->str ? cur->str : ""));
+    fprintf(stderr, "<b>(len %d) = %.*s\n",
+	    cur->len, cur->len, (cur->str ? cur->str : ""));
     for (i = 0; i < NARROWS; i++) {
 	cur = &(parse[i]);
-	printf("<%c>(len %d) = %.*s\n",
-	       Arrows[i].name,
-	       cur->len,
-	       cur->len,
-	       (cur->str ? cur->str : ""));
+	fprintf(stderr, "<%c>(len %d) = %.*s\n",
+		Arrows[i].name,
+		cur->len,
+		cur->len,
+		(cur->str ? cur->str : ""));
     }
     cur = &end;
-    printf("<e>(len %d) = %.*s\n",
-	   cur->len, cur->len, (cur->str ? cur->str : ""));
+    fprintf(stderr, "<e>(len %d) = %.*s\n",
+	    cur->len, cur->len, (cur->str ? cur->str : ""));
 #endif
 
     xtra_len = (beg.len + end.len);
@@ -361,29 +343,31 @@ menuarrow_add(char *string)
 
 	len = 0;
 	if (beg.len) {
-	    strncpy(str + len, beg.str, beg.len);
+	    STRNCPY(str + len, beg.str, beg.len);
 	    len += beg.len;
 	}
-	strncpy(str + len, parse[i].str, parse[i].len);
+	STRNCPY(str + len, parse[i].str, parse[i].len);
 	len += parse[i].len;
 
 	if (end.len) {
-	    strncpy(str + len, end.str, end.len);
+	    STRNCPY(str + len, end.str, end.len);
 	    len += end.len;
 	}
 	str[len] = '\0';
 
 #ifdef DEBUG_MENUARROWS
-	printf("<%c>(len %d) = %s\n", Arrows[i].name, len, str);
+	fprintf(stderr, "<%c>(len %d) = %s\n", Arrows[i].name, len, str);
 #endif
 	if (action_type(&(CurrentBar->arrows[i]), str) < 0)
 	    FREE(str);
     }
 }
+#endif
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 menuitem_t     *
-menuitem_add(menu_t * menu, char *name, char *name2, char *action)
+menuitem_add(menu_t *menu, const char *name, const char *name2, const char *action)
 {
     menuitem_t     *item;
     unsigned int    len;
@@ -420,7 +404,7 @@ menuitem_add(menu_t * menu, char *name, char *name2, char *action)
 	}
     }
 /* allocate a new itemect */
-    if ((item = (menuitem_t *)MALLOC(sizeof(menuitem_t))) == NULL)
+    if ((item = (menuitem_t *) MALLOC(sizeof(menuitem_t))) == NULL)
 	return NULL;
 
     item->len2 = 0;
@@ -429,7 +413,7 @@ menuitem_add(menu_t * menu, char *name, char *name2, char *action)
     len = strlen(name);
     item->name = MALLOC(len + 1);
     if (item->name != NULL) {
-	strcpy(item->name, name);
+	STRCPY(item->name, name);
 	if (name[0] == '.' && name[1] != '.')
 	    len = 0;		/* hidden menu name */
     } else {
@@ -459,7 +443,7 @@ menuitem_add(menu_t * menu, char *name, char *name2, char *action)
 	    len = 0;
 	    item->name2 = NULL;
 	} else {
-	    strcpy(item->name2, name2);
+	    STRCPY(item->name2, name2);
 	}
 	item->len2 = len;
     }
@@ -477,7 +461,7 @@ menuitem_add(menu_t * menu, char *name, char *name2, char *action)
 	    menuitem_free(menu, item);
 	    return NULL;
 	}
-	strcpy(str, action);
+	STRCPY(str, action);
 
 	if (action_type(&(item->entry.action), str) < 0)
 	    FREE(str);
@@ -488,14 +472,16 @@ menuitem_add(menu_t * menu, char *name, char *name2, char *action)
 
     return item;
 }
+#endif
 
+#ifdef MENUBAR
 /*
  * search for the base starting menu for NAME.
  * return a pointer to the portion of NAME that remains
  */
-/* PROTO */
+/* INTPROTO */
 char           *
-menu_find_base(menu_t ** menu, char *path)
+menu_find_base(menu_t **menu, char *path)
 {
     menu_t         *m = NULL;
     menuitem_t     *item;
@@ -566,13 +552,15 @@ menu_find_base(menu_t ** menu, char *path)
     }
     return path;
 }
+#endif
 
+#ifdef MENUBAR
 /*
  * delete this entire menu
  */
-/* PROTO */
+/* INTPROTO */
 menu_t         *
-menu_delete(menu_t * menu)
+menu_delete(menu_t *menu)
 {
     menu_t         *parent = NULL, *prev, *next;
     menuitem_t     *item;
@@ -629,10 +617,12 @@ menu_delete(menu_t * menu)
 
     return parent;
 }
+#endif
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 menu_t         *
-menu_add(menu_t * parent, char *path)
+menu_add(menu_t *parent, char *path)
 {
     menu_t         *menu;
 
@@ -662,7 +652,7 @@ menu_add(menu_t * parent, char *path)
 	return parent;
 
 /* allocate a new menu */
-    if ((menu = (menu_t *)MALLOC(sizeof(menu_t))) == NULL)
+    if ((menu = (menu_t *) MALLOC(sizeof(menu_t))) == NULL)
 	return parent;
 
     menu->width = 0;
@@ -673,7 +663,7 @@ menu_add(menu_t * parent, char *path)
 	FREE(menu);
 	return parent;
     }
-    strcpy(menu->name, path);
+    STRCPY(menu->name, path);
 
 /* initialize head/tail */
     menu->head = menu->tail = NULL;
@@ -708,8 +698,10 @@ menu_add(menu_t * parent, char *path)
 
     return menu;
 }
+#endif
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 void
 drawbox_menubar(int x, int len, int state)
 {
@@ -734,7 +726,7 @@ drawbox_menubar(int x, int len, int state)
 	top = botShadowGC;
 	bot = topShadowGC;
 	break;			/* SHADOW_IN */
-    case 0:
+    default:
 	top = bot = neutralGC;
 	break;			/* neutral */
     }
@@ -742,8 +734,10 @@ drawbox_menubar(int x, int len, int state)
     Draw_Shadow(menuBar.win, top, bot,
 		x, 0, len, menuBar_TotalHeight());
 }
+#endif
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 void
 drawtriangle(int x, int y, int state)
 {
@@ -762,20 +756,22 @@ drawtriangle(int x, int y, int state)
 	top = botShadowGC;
 	bot = topShadowGC;
 	break;			/* SHADOW_IN */
-    case 0:
+    default:
 	top = bot = neutralGC;
 	break;			/* neutral */
     }
 
-    w = menu_height() / 2;
+    w = Height2Pixel(1) - 2 * SHADOW;
 
-    x -= (SHADOW + MENU_MARGIN) + (3 * w / 2);
-    y += (SHADOW + MENU_MARGIN) + (w / 2);
+    x -= SHADOW + (3 * w / 2);
+    y += SHADOW * 3;
 
     Draw_Triangle(ActiveMenu->win, top, bot, x, y, w, 'r');
 }
+#endif
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 void
 drawbox_menuitem(int y, int state)
 {
@@ -793,7 +789,7 @@ drawbox_menuitem(int y, int state)
 	top = botShadowGC;
 	bot = topShadowGC;
 	break;			/* SHADOW_IN */
-    case 0:
+    default:
 	top = bot = neutralGC;
 	break;			/* neutral */
     }
@@ -802,20 +798,22 @@ drawbox_menuitem(int y, int state)
 		SHADOW + 0,
 		SHADOW + y,
 		ActiveMenu->w - 2 * (SHADOW),
-		menu_height() + 2 * MENU_MARGIN);
+		HEIGHT_TEXT + 2 * SHADOW);
     XFlush(Xdisplay);
 }
+#endif
 
 #ifdef DEBUG_MENU_LAYOUT
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 void
-print_menu_ancestors(menu_t * menu)
+print_menu_ancestors(menu_t *menu)
 {
     if (menu == NULL) {
-	printf("Top Level menu\n");
+	fprintf(stderr, "Top Level menu\n");
 	return;
     }
-    printf("menu %s ", menu->name);
+    fprintf(stderr, "menu %s ", menu->name);
     if (menu->parent != NULL) {
 	menuitem_t     *item;
 
@@ -826,17 +824,19 @@ print_menu_ancestors(menu_t * menu)
 	    }
 	}
 	if (item == NULL) {
-	    printf("is an orphan!\n");
+	    fprintf(stderr, "is an orphan!\n");
 	    return;
 	}
     }
-    printf("\n");
+    fprintf(stderr, "\n");
     print_menu_ancestors(menu->parent);
 }
+#endif
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 void
-print_menu_descendants(menu_t * menu)
+print_menu_descendants(menu_t *menu)
 {
     menuitem_t     *item;
     menu_t         *parent;
@@ -850,32 +850,34 @@ print_menu_descendants(menu_t * menu)
     while (parent != NULL);
 
     for (i = 0; i < level; i++)
-	printf(">");
-    printf("%s\n", menu->name);
+	fprintf(stderr, ">");
+    fprintf(stderr, "%s\n", menu->name);
 
     for (item = menu->head; item != NULL; item = item->next) {
 	if (item->entry.type == MenuSubMenu) {
 	    if (item->entry.submenu.menu == NULL)
-		printf("> %s == NULL\n", item->name);
+		fprintf(stderr, "> %s == NULL\n", item->name);
 	    else
 		print_menu_descendants(item->entry.submenu.menu);
 	} else {
 	    for (i = 0; i < level; i++)
-		printf("+");
+		fprintf(stderr, "+");
 	    if (item->entry.type == MenuLabel)
-		printf("label: ");
-	    printf("%s\n", item->name);
+		fprintf(stderr, "label: ");
+	    fprintf(stderr, "%s\n", item->name);
 	}
     }
 
     for (i = 0; i < level; i++)
-	printf("<");
-    printf("\n");
+	fprintf(stderr, "<");
+    fprintf(stderr, "\n");
 }
 #endif
+#endif
 
+#ifdef MENUBAR
 /* pop up/down the current menu and redraw the menuBar button */
-/* PROTO */
+/* INTPROTO */
 void
 menu_show(void)
 {
@@ -899,13 +901,10 @@ menu_show(void)
 	    x = (TermWin_TotalWidth() - ActiveMenu->w);
 
     /* find the height */
-	for (h = 0, item = ActiveMenu->head; item != NULL; item = item->next) {
-	    if (isSeparator(item->name))
-		h += SEPARATOR_HEIGHT;
-	    else
-		h += menu_height();
-	}
-	ActiveMenu->h = h + 2 * (SHADOW + MENU_MARGIN);
+	for (h = 0, item = ActiveMenu->head; item != NULL; item = item->next)
+	    h += isSeparator(item->name) ? HEIGHT_SEPARATOR
+					 : HEIGHT_TEXT + 2 * SHADOW;
+	ActiveMenu->h = h + 2 * SHADOW;
     }
     if (ActiveMenu->win == None) {
 	ActiveMenu->win = XCreateSimpleWindow(Xdisplay, TermWin.vt,
@@ -914,8 +913,8 @@ menu_show(void)
 					      ActiveMenu->w,
 					      ActiveMenu->h,
 					      0,
-					      PixColors[fgColor],
-					      PixColors[scrollColor]);
+					      PixColors[Color_fg],
+					      PixColors[Color_scroll]);
 	XMapWindow(Xdisplay, ActiveMenu->win);
     }
     Draw_Shadow(ActiveMenu->win,
@@ -928,22 +927,17 @@ menu_show(void)
 	if (item->len2 > xright)
 	    xright = item->len2;
 
-    for (y = 0, item = ActiveMenu->head;
-	 item != NULL;
-	 item = item->next) {
+    for (y = 0, item = ActiveMenu->head; item != NULL; item = item->next) {
 	const int       xoff = (SHADOW + Width2Pixel(HSPACE) / 2);
-	const int       yoff = (SHADOW + MENU_MARGIN);
 	register int    h;
 	GC              gc = menubarGC;
 
 	if (isSeparator(item->name)) {
 	    Draw_Shadow(ActiveMenu->win,
 			topShadowGC, botShadowGC,
-			xoff,
-			yoff + y + SEPARATOR_HALFHEIGHT,
-			ActiveMenu->w - (2 * xoff),
-			0);
-	    h = SEPARATOR_HEIGHT;
+			SHADOW, y + SHADOW + 1,
+			ActiveMenu->w - 2 * SHADOW, 0);
+	    h = HEIGHT_SEPARATOR;
 	} else {
 	    char           *name = item->name;
 	    int             len = item->len;
@@ -962,23 +956,19 @@ menu_show(void)
 
 		y1 = ActiveMenu->y + y;
 
-	    /* place sub-menu at midpoint of parent menu */
 		menu->w = Menu_PixelWidth(menu);
-		x1 = ActiveMenu->w / 2;
 
-	    /* right-flush menu if it's too small */
-		if (x1 > menu->w)
+	    /* place sub-menu at midpoint of parent menu */
+		x1 = ActiveMenu->w / 2;
+		if (x1 > menu->w)	/* right-flush menu if too small */
 		    x1 += (x1 - menu->w);
 		x1 += x;
 
 	    /* find the height of this submenu */
-		for (h = 0, it = menu->head; it != NULL; it = it->next) {
-		    if (isSeparator(it->name))
-			h += SEPARATOR_HEIGHT;
-		    else
-			h += menu_height();
-		}
-		menu->h = h + 2 * (SHADOW + MENU_MARGIN);
+		for (h = 0, it = menu->head; it != NULL; it = it->next)
+		    h += isSeparator(it->name) ? HEIGHT_SEPARATOR
+					       : HEIGHT_TEXT + 2 * SHADOW;
+		menu->h = h + 2 * SHADOW;
 
 	    /* ensure menu is in window limits */
 		if ((x1 + menu->w) >= TermWin.width)
@@ -996,7 +986,7 @@ menu_show(void)
 		XDrawString(Xdisplay,
 			    ActiveMenu->win, gc,
 			    xoff,
-			    yoff + y + menu_height() - (2 * MENU_MARGIN),
+			    2 * SHADOW + y + TermWin.font->ascent + 1,
 			    name, len);
 
 	    len = item->len2;
@@ -1005,16 +995,18 @@ menu_show(void)
 		XDrawString(Xdisplay,
 			    ActiveMenu->win, gc,
 			    ActiveMenu->w - (xoff + Width2Pixel(xright)),
-			    yoff + y + menu_height() - (2 * MENU_MARGIN),
+			    2 * SHADOW + y + TermWin.font->ascent + 1,
 			    name, len);
 
-	    h = menu_height();
+	    h = HEIGHT_TEXT + 2 * SHADOW;
 	}
 	y += h;
     }
 }
+#endif
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 void
 menu_display(void (*update) (void))
 {
@@ -1030,24 +1022,30 @@ menu_display(void (*update) (void))
     ActiveMenu = ActiveMenu->parent;
     update();
 }
+#endif
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 void
 menu_hide_all(void)
 {
     menu_display(menu_hide_all);
 }
+#endif
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 void
 menu_hide(void)
 {
     menu_display(menu_show);
 }
+#endif
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 void
-menu_clear(menu_t * menu)
+menu_clear(menu_t *menu)
 {
     if (menu != NULL) {
 	menuitem_t     *item = menu->tail;
@@ -1062,8 +1060,10 @@ menu_clear(menu_t * menu)
 	menu->width = 0;
     }
 }
+#endif
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 void
 menubar_clear(void)
 {
@@ -1086,17 +1086,19 @@ menubar_clear(void)
     }
     ActiveMenu = NULL;
 }
+#endif
 
 #if (MENUBAR_MAX > 1)
+#ifdef MENUBAR
 /* find if menu already exists */
-/* PROTO */
+/* INTPROTO */
 bar_t          *
 menubar_find(const char *name)
 {
     bar_t          *bar = CurrentBar;
 
 #ifdef DEBUG_MENUBAR_STACKING
-    printf("looking for [menu:%s] ...", name ? name : "(nil)");
+    fprintf(stderr, "looking for [menu:%s] ...", name ? name : "(nil)");
 #endif
     if (bar == NULL || name == NULL)
 	return NULL;
@@ -1105,7 +1107,7 @@ menubar_find(const char *name)
 	do {
 	    if (!strcmp(bar->name, name)) {
 #ifdef DEBUG_MENUBAR_STACKING
-		printf(" found!\n");
+		fprintf(stderr, " found!\n");
 #endif
 		return bar;
 	    }
@@ -1115,13 +1117,15 @@ menubar_find(const char *name)
 	bar = NULL;
     }
 #ifdef DEBUG_MENUBAR_STACKING
-    printf("%s found!\n", (bar ? "" : " NOT"));
+    fprintf(stderr, "%s found!\n", (bar ? "" : " NOT"));
 #endif
 
     return bar;
 }
+#endif
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 int
 menubar_push(const char *name)
 {
@@ -1130,12 +1134,12 @@ menubar_push(const char *name)
 
     if (CurrentBar == NULL) {
     /* allocate first one */
-	bar = (bar_t *)MALLOC(sizeof(bar_t));
+	bar = (bar_t *) MALLOC(sizeof(bar_t));
 
 	if (bar == NULL)
 	    return 0;
 
-        rmemset(bar, 0, sizeof (bar_t));
+	MEMSET(bar, 0, sizeof(bar_t));
     /* circular linked-list */
 	bar->next = bar->prev = bar;
 	bar->head = bar->tail = NULL;
@@ -1155,7 +1159,7 @@ menubar_push(const char *name)
 	    if (CurrentBar->head != NULL) {
 	    /* need to malloc another one */
 		if (Nbars < MENUBAR_MAX)
-		    bar = (bar_t *)MALLOC(sizeof(bar_t));
+		    bar = (bar_t *) MALLOC(sizeof(bar_t));
 		else
 		    bar = NULL;
 
@@ -1182,14 +1186,16 @@ menubar_push(const char *name)
     }
 
 /* give menubar this name */
-    strncpy(CurrentBar->name, name, MAXNAME);
+    STRNCPY(CurrentBar->name, name, MAXNAME);
     CurrentBar->name[MAXNAME - 1] = '\0';
 
     return ret;
 }
+#endif
 
+#ifdef MENUBAR
 /* switch to a menu called NAME and remove it */
-/* PROTO */
+/* INTPROTO */
 void
 menubar_remove(const char *name)
 {
@@ -1223,10 +1229,12 @@ menubar_remove(const char *name)
     }
     while (CurrentBar && !strcmp(name, "*"));
 }
+#endif
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 void
-action_decode(FILE * fp, action_t * act)
+action_decode(FILE *fp, action_t *act)
 {
     unsigned char  *str;
     short           len;
@@ -1249,7 +1257,7 @@ action_decode(FILE * fp, action_t * act)
 	/* can strip trailing '\r' from M-x sequence */
 	    if (str[len - 1] == '\r')
 		len--;
-	/* drop */
+	/* FALLTHROUGH */
 
 	default:
 	    fprintf(fp, "M-");	/* meta prefix */
@@ -1295,10 +1303,12 @@ action_decode(FILE * fp, action_t * act)
     }
     fprintf(fp, "\n");
 }
+#endif
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 void
-menu_dump(FILE * fp, menu_t * menu)
+menu_dump(FILE *fp, menu_t *menu)
 {
     menuitem_t     *item;
 
@@ -1332,10 +1342,12 @@ menu_dump(FILE * fp, menu_t * menu)
 
     fprintf(fp, (menu->parent ? "../\n" : "/\n\n"));
 }
+#endif
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 void
-menubar_dump(FILE * fp)
+menubar_dump(FILE *fp)
 {
     bar_t          *bar = CurrentBar;
     time_t          t;
@@ -1346,7 +1358,7 @@ menubar_dump(FILE * fp)
 
     fprintf(fp,
 	    "# " APL_SUBCLASS " (%s)  Pid: %u\n# Date: %s\n\n",
-	    rs_name, (unsigned int)getpid(), ctime(&t));
+	    rs[Rs_name], (unsigned int)getpid(), ctime(&t));
 
 /* dump in reverse order */
     bar = CurrentBar->prev;
@@ -1378,6 +1390,7 @@ menubar_dump(FILE * fp)
     }
     while (bar != CurrentBar->prev);
 }
+#endif
 #endif				/* (MENUBAR_MAX > 1) */
 
 /*
@@ -1395,18 +1408,22 @@ menubar_dump(FILE * fp)
  * FILENAME = "file;tag"
  *      read `file' starting with [menu:tag]
  */
-/* PROTO */
+/* EXTPROTO */
 void
 menubar_read(const char *filename)
 {
+#ifdef MENUBAR
 /* read in a menu from a file */
     FILE           *fp;
     char            buffer[256];
-    char           *p, *tag = NULL;
-    const char     *file;
+    char           *p, *file, *tag = NULL;
 
-    file = File_find(filename, ".menu");
-    if (file == NULL || (fp = fopen(file, "rb")) == NULL)
+    file = (char *) File_find(filename, ".menu");
+    if (file == NULL)
+	return;
+    fp = fopen(file, "rb");
+    FREE(file);
+    if (fp == NULL)
 	return;
 
 #if (MENUBAR_MAX > 1)
@@ -1418,9 +1435,9 @@ menubar_read(const char *filename)
     }
 #endif				/* (MENUBAR_MAX > 1) */
 #ifdef DEBUG_MENU
-    printf("[read:%s]\n", p);
+    fprintf(stderr, "[read:%s]\n", p);
     if (tag)
-	printf("looking for [menu:%s]\n", tag);
+	fprintf(stderr, "looking for [menu:%s]\n", tag);
 #endif
 
     while ((p = fgets(buffer, sizeof(buffer), fp)) != NULL) {
@@ -1434,7 +1451,7 @@ menubar_read(const char *filename)
 		    n += Str_match(p + n, tag);
 		    if (p[n] == ']') {
 #ifdef DEBUG_MENU
-			printf("[menu:%s]\n", tag);
+			fprintf(stderr, "[menu:%s]\n", tag);
 #endif
 			break;
 		    }
@@ -1449,7 +1466,7 @@ menubar_read(const char *filename)
 	int             n;
 
 #ifdef DEBUG_MENU
-	printf("read line = %s\n", p);
+	fprintf(stderr, "read line = %s\n", p);
 #endif
 
     /* looking for [done:tag] or [done:] */
@@ -1466,7 +1483,7 @@ menubar_read(const char *filename)
 		    n += Str_match(p + n, tag);
 		    if (p[n] == ']') {
 #ifdef DEBUG_MENU
-			printf("[done:%s]\n", tag);
+			fprintf(stderr, "[done:%s]\n", tag);
 #endif
 			menu_readonly = 1;
 			break;
@@ -1482,8 +1499,8 @@ menubar_read(const char *filename)
      * and strip-off leading/trailing quotes
      * skip blank or comment lines
      */
-	p = Str_trim(p);
-	if (p != NULL && *p && *p != '#') {
+	(void) Str_trim(p);
+	if (*p && *p != '#') {
 	    menu_readonly = 0;	/* if case we read another file */
 	    menubar_dispatch(p);
 	}
@@ -1492,15 +1509,17 @@ menubar_read(const char *filename)
     }
 
     fclose(fp);
+#endif
 }
 
 /*
  * user interface for building/deleting and otherwise managing menus
  */
-/* PROTO */
+/* EXTPROTO */
 void
 menubar_dispatch(char *str)
 {
+#ifdef MENUBAR
     static menu_t  *BuildMenu = NULL;	/* the menu currently being built */
     int             n, cmd;
     char           *path, *name, *name2;
@@ -1583,9 +1602,9 @@ menubar_dispatch(char *str)
 		str += n;
 		if (CurrentBar != NULL && !menu_readonly) {
 		    if (*str) {
-			name = REMALLOC(CurrentBar->title, strlen(str) + 1);
+			name = REALLOC(CurrentBar->title, strlen(str) + 1);
 			if (name != NULL) {
-			    strcpy(name, str);
+			    STRCPY(name, str);
 			    CurrentBar->title = name;
 			}
 			menubar_expose();
@@ -1625,12 +1644,11 @@ menubar_dispatch(char *str)
 		    if (*str != '\0' && *str != '*')
 			menubar_push(str);
 		    break;
-                default:
-                    if (CurrentBar == NULL) {
-                        menubar_push("default");
-                    }
+		default:
+		    if (CurrentBar == NULL) {
+			menubar_push("default");
+		    }
 		}
-                    
 
 		if (CurrentBar != NULL)
 		    menu_readonly = 0;	/* allow menu build commands */
@@ -1684,7 +1702,7 @@ menubar_dispatch(char *str)
 	    BuildMenu = ActiveMenu = NULL;
 	    menubar_expose();
 #ifdef DEBUG_MENUBAR_STACKING
-	    printf("menus are read%s\n", menu_readonly ? "only" : "/write");
+	    fprintf(stderr, "menus are read%s\n", menu_readonly ? "only" : "/write");
 #endif
 	}
 	return;
@@ -1696,7 +1714,7 @@ menubar_dispatch(char *str)
 	return;
     if (menu_readonly) {
 #ifdef DEBUG_MENUBAR_STACKING
-	printf("menus are read%s\n", menu_readonly ? "only" : "/write");
+	fprintf(stderr, "menus are read%s\n", menu_readonly ? "only" : "/write");
 #endif
 	return;
     }
@@ -1738,12 +1756,12 @@ menubar_dispatch(char *str)
 		    str++;	/* skip space */
 	    }
 #ifdef DEBUG_MENU
-	    printf("`%c' path = <%s>, name = <%s>, name2 = <%s>, action = <%s>\n",
-		   cmd,
-		   (path ? path : "(nil)"),
-		   (name ? name : "(nil)"),
-		   (name2 ? name2 : "(nil)"),
-		   (str ? str : "(nil)")
+	    fprintf(stderr, "`%c' path = <%s>, name = <%s>, name2 = <%s>, action = <%s>\n",
+		    cmd,
+		    (path ? path : "(nil)"),
+		    (name ? name : "(nil)"),
+		    (name2 ? name2 : "(nil)"),
+		    (str ? str : "(nil)")
 		);
 #endif
 	}
@@ -1766,11 +1784,10 @@ menubar_dispatch(char *str)
 		if (path[0] != '\0')
 		    BuildMenu = menu_add(BuildMenu, path);
 	    }
-	    if (name != NULL && name[0] != '\0') {
-		if (!strcmp(name, SEPARATOR_NAME))
-		    name = "";
-		menuitem_add(BuildMenu, name, name2, str);
-	    }
+	    if (name != NULL && name[0] != '\0')
+		menuitem_add(BuildMenu,
+			     (strcmp(name, SEPARATOR_NAME) ? name : ""),
+			     name2, str);
 	    break;
 
 	case '-':		/* delete menu entry */
@@ -1806,9 +1823,9 @@ menubar_dispatch(char *str)
 		} else {
 		    menuitem_t     *item;
 
-		    if (!strcmp(name, SEPARATOR_NAME))
-			name = "";
-		    item = menuitem_find(BuildMenu, name);
+		    item = menuitem_find(BuildMenu,
+				         strcmp(name, SEPARATOR_NAME) ? name
+								      : "");
 
 		    if (item != NULL && item->entry.type != MenuSubMenu) {
 			menuitem_free(BuildMenu, item);
@@ -1829,9 +1846,11 @@ menubar_dispatch(char *str)
 	}
 	break;
     }
+#endif
 }
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 void
 draw_Arrows(int name, int state)
 {
@@ -1851,7 +1870,7 @@ draw_Arrows(int name, int state)
 	top = botShadowGC;
 	bot = topShadowGC;
 	break;			/* SHADOW_IN */
-    case 0:
+    default:
 	top = bot = neutralGC;
 	break;			/* neutral */
     }
@@ -1870,15 +1889,17 @@ draw_Arrows(int name, int state)
     }
     XFlush(Xdisplay);
 }
+#endif
 
-/* PROTO */
+/* EXTPROTO */
 void
 menubar_expose(void)
 {
+#ifdef MENUBAR
     menu_t         *menu;
     int             x;
 
-    if (delay_menu_drawing || !menubar_visible())
+    if (!menubar_visible() || menuBar.win == 0)
 	return;
 
     if (menubarGC == None) {
@@ -1888,23 +1909,23 @@ menubar_expose(void)
 	gcvalue.font = TermWin.font->fid;
 
 	gcvalue.foreground = (Xdepth <= 2 ?
-			      PixColors[fgColor] :
-			      PixColors[BlackColor]);
+			      PixColors[Color_fg] :
+			      PixColors[Color_Black]);
 	menubarGC = XCreateGC(Xdisplay, menuBar.win,
 			      GCForeground | GCFont,
 			      &gcvalue);
 
-	gcvalue.foreground = PixColors[scrollColor];
+	gcvalue.foreground = PixColors[Color_scroll];
 	neutralGC = XCreateGC(Xdisplay, menuBar.win,
 			      GCForeground,
 			      &gcvalue);
 
-	gcvalue.foreground = PixColors[bottomShadowColor];
+	gcvalue.foreground = PixColors[Color_bottomShadow];
 	botShadowGC = XCreateGC(Xdisplay, menuBar.win,
 				GCForeground | GCFont,
 				&gcvalue);
 
-	gcvalue.foreground = PixColors[topShadowColor];
+	gcvalue.foreground = PixColors[Color_topShadow];
 	topShadowGC = XCreateGC(Xdisplay, menuBar.win,
 				GCForeground,
 				&gcvalue);
@@ -1934,8 +1955,8 @@ menubar_expose(void)
 
 	    XDrawString(Xdisplay,
 			menuBar.win, menubarGC,
-			(Width2Pixel(menu->x) + Width2Pixel(HSPACE) / 2 ),
-			menuBar_height() - MENU_MARGIN,
+			(Width2Pixel(menu->x) + Width2Pixel(HSPACE) / 2),
+			menuBar_height() - SHADOW,
 			menu->name, len);
 
 	    if (x >= TermWin.ncol)
@@ -1947,8 +1968,9 @@ menubar_expose(void)
 /* add the menuBar title, if it exists and there's plenty of room */
     Arrows_x = 0;
     if (x < TermWin.ncol) {
-	char           *str, title[256];
+	const char     *str;
 	int             len, ncol = TermWin.ncol;
+	char            title[256];
 
 	if (x < (ncol - (NARROWS + 1))) {
 	    ncol -= (NARROWS + 1);
@@ -1956,7 +1978,8 @@ menubar_expose(void)
 	}
 	draw_Arrows(0, +1);
 
-	str = (CurrentBar && CurrentBar->title ? CurrentBar->title : "%n-%v");
+	str = (CurrentBar && CurrentBar->title) ? CurrentBar->title
+						: "%n-%v";
 	for (len = 0; str[0] && len < sizeof(title) - 1; str++) {
 	    const char     *s = NULL;
 
@@ -1965,7 +1988,7 @@ menubar_expose(void)
 		str++;
 		switch (str[0]) {
 		case 'n':
-		    s = rs_name;
+		    s = rs[Rs_name];
 		    break;	/* resource name */
 		case 'v':
 		    s = VERSION;
@@ -1991,19 +2014,23 @@ menubar_expose(void)
 	    XDrawString(Xdisplay,
 			menuBar.win, menubarGC,
 			Width2Pixel(x) + Width2Pixel(ncol + HSPACE) / 2,
-			menuBar_height() - MENU_MARGIN,
+			menuBar_height() - SHADOW,
 			title, len);
     }
+#endif
 }
 
-/* PROTO */
+/* EXTPROTO */
 int
 menubar_mapping(int map)
 {
+#ifdef MENUBAR
     int             change = 0;
 
     if (map && !menubar_visible()) {
 	menuBar.state = 1;
+	if (menuBar.win == 0)
+	    return 0;
 	XMapWindow(Xdisplay, menuBar.win);
 	change = 1;
     } else if (!map && menubar_visible()) {
@@ -2015,11 +2042,13 @@ menubar_mapping(int map)
 	menubar_expose();
 
     return change;
+#endif
 }
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 int
-menu_select(XButtonEvent * ev)
+menu_select(XButtonEvent *ev)
 {
     menuitem_t     *thisitem, *item = NULL;
     int             this_y, y;
@@ -2042,18 +2071,16 @@ menu_select(XButtonEvent * ev)
 	return 1;
     }
 /* determine the menu item corresponding to the Y index */
+    y = SHADOW;
     if (ev->x >= 0 &&
 	ev->x <= (ActiveMenu->w - SHADOW)) {
-	for (y = 0, item = ActiveMenu->head;
-	     item != NULL;
-	     item = item->next) {
-	    int             h = menu_height();
+	for (item = ActiveMenu->head; item != NULL; item = item->next) {
+	    int             h = HEIGHT_TEXT + 2 * SHADOW;
 
-	    if (isSeparator(item->name)) {
-		h = SEPARATOR_HEIGHT;
-	    } else if (ev->y >= y && ev->y < (y + h)) {
+	    if (isSeparator(item->name))
+		h = HEIGHT_SEPARATOR;
+	    else if (ev->y >= y && ev->y < (y + h))
 		break;
-	    }
 	    y += h;
 	}
     }
@@ -2062,7 +2089,7 @@ menu_select(XButtonEvent * ev)
 	return 0;
     }
     thisitem = item;
-    this_y = y;
+    this_y = y - SHADOW;
 
 /* erase the last item */
     if (ActiveMenu->item != NULL) {
@@ -2070,17 +2097,18 @@ menu_select(XButtonEvent * ev)
 	    for (y = 0, item = ActiveMenu->head;
 		 item != NULL;
 		 item = item->next) {
-		int             h = menu_height();
+		int             h;
 
-		if (isSeparator(item->name)) {
-		    h = SEPARATOR_HEIGHT;
-		} else if (item == ActiveMenu->item) {
+		if (isSeparator(item->name))
+		    h = HEIGHT_SEPARATOR;
+		else if (item == ActiveMenu->item) {
 		/* erase old menuitem */
 		    drawbox_menuitem(y, 0);	/* No Shadow */
 		    if (item->entry.type == MenuSubMenu)
 			drawtriangle(ActiveMenu->w, y, +1);
 		    break;
-		}
+		} else
+		    h = HEIGHT_TEXT + 2 * SHADOW;
 		y += h;
 	    }
 	} else {
@@ -2100,18 +2128,25 @@ menu_select(XButtonEvent * ev)
 		 * remove menu before sending keys to the application
 		 */
 		    {
+#ifdef __CYGWIN32__
+			struct timeval  tv;
+
+			tv.tv_sec = 0;
+			tv.tv_usec = MENU_DELAY_USEC;
+			select(0, NULL, NULL, NULL, &tv);
+#else
 			struct itimerval tv;
 
 			tv.it_value.tv_sec = 0;
 			tv.it_value.tv_usec = MENU_DELAY_USEC;
-
 			select(0, NULL, NULL, NULL, &tv.it_value);
+#endif
 		    }
 		    menu_hide_all();
 #ifndef DEBUG_MENU
 		    action_dispatch(&(item->entry.action));
 #else				/* DEBUG_MENU */
-		    printf("%s: %s\n", item->name, item->entry.action.str);
+		    fprintf(stderr, "%s: %s\n", item->name, item->entry.action.str);
 #endif				/* DEBUG_MENU */
 		    break;
 		}
@@ -2150,10 +2185,12 @@ menu_select(XButtonEvent * ev)
     }
     return 0;
 }
+#endif
 
-/* PROTO */
+#ifdef MENUBAR
+/* INTPROTO */
 void
-menubar_select(XButtonEvent * ev)
+menubar_select(XButtonEvent *ev)
 {
     menu_t         *menu = NULL;
 
@@ -2184,25 +2221,32 @@ menubar_select(XButtonEvent * ev)
 		 * use select for timing
 		 */
 		    {
+#ifdef __CYGWIN32__
+			struct timeval  tv;
+
+			tv.tv_sec = 0;
+			tv.tv_usec = MENU_DELAY_USEC;
+			select(0, NULL, NULL, NULL, &tv);
+#else
 			struct itimerval tv;
 
 			tv.it_value.tv_sec = 0;
 			tv.it_value.tv_usec = MENU_DELAY_USEC;
-
 			select(0, NULL, NULL, NULL, &tv.it_value);
+#endif
 		    }
 		    draw_Arrows(Arrows[i].name, +1);
 #ifdef DEBUG_MENUARROWS
-		    printf("'%c': ", Arrows[i].name);
+		    fprintf(stderr, "'%c': ", Arrows[i].name);
 
 		    if (CurrentBar == NULL ||
 			(CurrentBar->arrows[i].type != MenuAction &&
 			 CurrentBar->arrows[i].type != MenuTerminalAction)) {
 			if (Arrows[i].str != NULL && Arrows[i].str[0])
-			    printf("(default) \\033%s\n",
-				   &(Arrows[i].str[2]));
+			    fprintf(stderr, "(default) \\033%s\n",
+				    &(Arrows[i].str[2]));
 		    } else {
-			printf("%s\n", CurrentBar->arrows[i].str);
+			fprintf(stderr, "%s\n", CurrentBar->arrows[i].str);
 		    }
 #else				/* DEBUG_MENUARROWS */
 		    if (CurrentBar == NULL ||
@@ -2217,7 +2261,7 @@ menubar_select(XButtonEvent * ev)
 		}
 	    }
 	}
-    /*drop */
+    /* FALLTHROUGH */
 
     default:
     /*
@@ -2231,15 +2275,17 @@ menubar_select(XButtonEvent * ev)
 	break;
     }
 }
+#endif
 
 /*
  * general dispatch routine,
  * it would be nice to have `sticky' menus
  */
-/* PROTO */
+/* EXTPROTO */
 void
-menubar_control(XButtonEvent * ev)
+menubar_control(XButtonEvent *ev)
 {
+#ifdef MENUBAR
     switch (ev->type) {
     case ButtonPress:
 	if (ev->button == Button1)
@@ -2274,6 +2320,46 @@ menubar_control(XButtonEvent * ev)
 	}
 	break;
     }
+#endif
 }
-#endif				/* (MENUBAR_MAX) */
+
+/* EXTPROTO */
+void
+map_menuBar(int map)
+{
+#ifdef MENUBAR
+    if (menubar_mapping(map))
+	resize_all_windows();
+#endif
+}
+
+/* EXTPROTO */
+void
+create_menuBar(Cursor cursor)
+{
+#ifdef MENUBAR
+/* menuBar: size doesn't matter */
+    menuBar.win = XCreateSimpleWindow(Xdisplay, TermWin.parent,
+				      0, 0,
+				      1, 1,
+				      0,
+				      PixColors[Color_fg],
+				      PixColors[Color_scroll]);
+    XDefineCursor(Xdisplay, menuBar.win, cursor);
+    XSelectInput(Xdisplay, menuBar.win,
+		 (ExposureMask | ButtonPressMask | ButtonReleaseMask |
+		  Button1MotionMask));
+
+#endif
+}
+
+/* EXTPROTO */
+void
+Resize_menuBar(int x, int y, unsigned int width, unsigned int height)
+{
+#ifdef MENUBAR
+    XMoveResizeWindow(Xdisplay, menuBar.win, x, y, width, height);
+#endif
+}
+
 /*----------------------- end-of-file (C source) -----------------------*/
