@@ -1,12 +1,10 @@
 /*
- * $Date: Sun, 03 Aug 1997 16:47:17 +0000 $
- *
+ * $Id: rxvt.h,v 1.29 1998/11/17 19:44:10 mason Exp $
  */
 
 #ifndef _RXVT_H			/* include once only */
 #define _RXVT_H
 
-#include "version.h"
 #include "../config.h"
 #include "feature.h"
 
@@ -60,40 +58,31 @@
 #ifdef HAVE_STDARG_H
 # include <stdarg.h>
 #endif
-
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
-
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif
-
 #ifdef HAVE_FCNTL_H
 # include <fcntl.h>
 #endif
-
 #ifdef HAVE_SYS_TYPES_H
 # include <sys/types.h>
 #endif
-
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
-
 #ifdef HAVE_UTIL_H
 # include <util.h>
 #endif
-
 #ifdef HAVE_ASSERT_H
 # include <assert.h>
 #endif
-
 #if defined (HAVE_SYS_IOCTL_H) && !defined (__sun__)
 /* seems to cause problems when <termios.h> is also included on some suns */
 # include <sys/ioctl.h>
 #endif
-
 #ifdef TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
@@ -104,50 +93,39 @@
 #  include <time.h>
 # endif
 #endif
-
 #ifdef HAVE_SYS_SELECT_H
 # include <sys/select.h>
 #endif
 
-#ifdef HAVE_TERMIOS_H
-# include <termios.h>
-#else
-# include <sgtty.h>
-#endif
-
-#ifdef UTMP_SUPPORT
-# ifdef HAVE_UTMPX_H
-#  include <utmpx.h>
-#  define USE_SYSV_UTMP
-# else
-#  include <utmp.h>
-#  ifdef HAVE_SETUTENT
-#   define USE_SYSV_UTMP
-#  endif
-# endif
-# ifdef HAVE_LASTLOG_H
-#  include <lastlog.h>
-# endif
-# include <pwd.h>
-#endif
-
 #include <sys/wait.h>
 #include <sys/stat.h>
 
 #include <X11/Intrinsic.h>	/* Xlib, Xutil, Xresource, Xfuncproto */
 #include <X11/cursorfont.h>
 #include <X11/keysym.h>
-#ifdef XPM_BACKGROUND
-# ifdef XPM_INC_X11
-#  include <X11/xpm.h>
-# else
-#  include <xpm.h>
-# endif
-#endif
 
 #ifdef RXVT_GRAPHICS
 # include "rxvtgrx.h"
 #endif
+#ifdef GREEK_SUPPORT
+# include "grkelot.h"
+#endif
+#if defined(KANJI) || defined(ZH)
+# define MULTICHAR_SET		/* a glyph is defined by 1 or 2 chars */
+#else
+# undef  MULTICHAR_SET		/* a glyph is only ever defined by 1 char */
+#endif
+
+/*
+ * Play some size tricks:  <X11/Xmd.h> might be better
+ * Now if only we could presume every OS has int16_t, u_int16_t, etc.
+ */
+typedef RINT16T		R_int16_t;
+typedef RUINT16T	R_u_int16_t;
+typedef RINT32T		R_int32_t;
+typedef RUINT32T	R_u_int32_t;
+typedef RINTPT		R_int_p_t;
+typedef RUINTPT		R_u_int_p_t;
 
 /*
  *****************************************************************************
@@ -162,6 +140,11 @@
 # define MENUBAR_MAX	0
 #endif
 
+typedef struct {
+    short           state;
+    Window          win;
+} menuBar_t;
+
 /* If we're using either the fancy scrollbar or menu bars, keep the
  * scrollColor resource.
  */
@@ -172,27 +155,24 @@
 #endif
 
 typedef struct {
-    short           width,	/* window width [pixels]                    */
-		    height;	/* window height [pixels]                   */
-    short           fwidth,	/* font width [pixels]                      */
-		    fheight;	/* font height [pixels]                     */
-    short	    fprop;	/* font is proportional                     */
-    short           ncol, nrow;	/* window size [characters]                 */
-    short           focus;	/* window has focus                         */
-    short           saveLines;	/* number of lines that fit in scrollback   */
-    short           nscrolled;	/* number of line actually scrolled         */
-    short           view_start;	/* scrollback view starts here              */
+    R_int16_t       width,	/* window width [pixels]                    */
+		    height,	/* window height [pixels]                   */
+		    fwidth,	/* font width [pixels]                      */
+		    fheight,	/* font height [pixels]                     */
+		    fprop,	/* font is proportional                     */
+		    ncol, nrow,	/* window size [characters]                 */
+		    focus,	/* window has focus                         */
+		    saveLines,	/* number of lines that fit in scrollback   */
+		    nscrolled,	/* number of line actually scrolled         */
+		    view_start;	/* scrollback view starts here              */
     Window          parent, vt;	/* parent (main) and vt100 window           */
     GC              gc;		/* GC for drawing text                      */
     XFontStruct    *font;	/* main font structure                      */
 #ifndef NO_BOLDFONT
     XFontStruct    *boldFont;	/* bold font                                */
 #endif
-#ifdef KANJI
-    XFontStruct    *kanji;	/* Kanji font structure                     */
-#endif
-#ifdef BIG5
-    XFontStruct    *big5;	/* Chinese Big5 font structure */
+#ifdef MULTICHAR_SET
+    XFontStruct    *mfont;	/* Multichar font structure                 */
 #endif
 #ifdef XPM_BACKGROUND
     Pixmap          pixmap;
@@ -209,72 +189,7 @@ typedef struct {
     Window          win;
 } scrollBar_t;
 
-typedef struct {
-    short           state;
-    Window          win;
-} menuBar_t;
 
-typedef struct {
-    short           type;	/* must not be changed; first element */
-    short           len;	/* strlen (str) */
-    unsigned char  *str;	/* action to take */
-} action_t;
-
-typedef struct {
-    short           type;	/* must not be changed; first element */
-    struct menu_t  *menu;	/* sub-menu */
-} submenu_t;
-
-typedef struct menuitem_t {
-    struct menuitem_t *prev;	/* prev menu-item */
-    struct menuitem_t *next;	/* next menu-item */
-    char           *name;	/* character string displayed */
-    char           *name2;	/* character string displayed (right) */
-    short           len;	/* strlen (name) */
-    short           len2;	/* strlen (name) */
-    union {
-	short           type;	/* must not be changed; first element */
-	action_t        action;
-	submenu_t       submenu;
-    } entry;
-} menuitem_t;
-
-enum menuitem_t_action {
-    MenuLabel,
-    MenuAction,
-    MenuTerminalAction,
-    MenuSubMenu
-};
-
-typedef struct menu_t {
-    struct menu_t  *parent;	/* parent menu */
-    struct menu_t  *prev;	/* prev menu */
-    struct menu_t  *next;	/* next menu */
-    menuitem_t     *head;	/* double-linked list */
-    menuitem_t     *tail;	/* double-linked list */
-    menuitem_t     *item;	/* current item */
-    char           *name;	/* menu name */
-    short           len;	/* strlen (name) */
-    short           width;	/* maximum menu width [chars] */
-    Window          win;	/* window of the menu */
-    short           x;		/* x location [pixels] (chars if parent == NULL) */
-    short           y;		/* y location [pixels] */
-    short           w, h;	/* window width, height [pixels] */
-} menu_t;
-
-typedef struct bar_t {
-    menu_t         *head, *tail;	/* double-linked list of menus */
-    char           *title;	/* title to put in the empty menuBar */
-#if (MENUBAR_MAX > 1)
-# define MAXNAME 16
-    char            name[MAXNAME];	/* name to use to refer to menubar */
-    struct bar_t   *next, *prev;	/* circular linked-list */
-#endif				/* (MENUBAR_MAX > 1) */
-#define NARROWS	4
-    action_t        arrows[NARROWS];
-} bar_t;
-
-#ifdef RXVT_GRAPHICS
 typedef struct grcmd_t {
     char            cmd;
     short           color;
@@ -292,59 +207,12 @@ typedef struct grwin_t {
     grcmd_t        *graphics;
     struct grwin_t *prev, *next;
 } grwin_t;
-#endif
 
 typedef struct {
-    int             row, col;
+    R_int16_t       row, col;
 } row_col_t;
 
 
-/* ways to deal with getting/setting termios structure */
-#ifdef HAVE_TERMIOS_H
-typedef struct termios ttymode_t;
-
-# ifdef TCSANOW			/* POSIX */
-#  define GET_TERMIOS(fd,tios)	tcgetattr (fd, tios)
-#  define SET_TERMIOS(fd,tios)		\
-	cfsetospeed (tios, BAUDRATE),	\
-	cfsetispeed (tios, BAUDRATE),	\
-	tcsetattr (fd, TCSANOW, tios)
-# else
-#  ifdef TIOCSETA
-#   define GET_TERMIOS(fd,tios)	ioctl (fd, TIOCGETA, tios)
-#   define SET_TERMIOS(fd,tios)		\
-	tios->c_cflag |= BAUDRATE,	\
-	ioctl (fd, TIOCSETA, tios)
-#  else
-#   define GET_TERMIOS(fd,tios)	ioctl (fd, TCGETS, tios)
-#   define SET_TERMIOS(fd,tios)		\
-	tios->c_cflag |= BAUDRATE,	\
-	ioctl (fd, TCSETS, tios)
-#  endif
-# endif
-# define SET_TTYMODE(fd,tios)		SET_TERMIOS (fd, tios)
-#else
-/* sgtty interface */
-typedef struct {
-    struct sgttyb   sg;
-    struct tchars   tc;
-    struct ltchars  lc;
-    int             line;
-    int             local;
-} ttymode_t;
-
-# define SET_TTYMODE(fd,tt)				\
-	tt->sg.sg_ispeed = tt->sg.sg_ospeed = BAUDRATE,	\
-	ioctl (fd, TIOCSETP, &(tt->sg)),		\
-	ioctl (fd, TIOCSETC, &(tt->tc)),		\
-	ioctl (fd, TIOCSLTC, &(tt->lc)),		\
-	ioctl (fd, TIOCSETD, &(tt->line)),		\
-	ioctl (fd, TIOCLSET, &(tt->local))
-#endif				/* HAVE_TERMIOS_H */
-
-typedef unsigned char text_t;
-typedef unsigned int rend_t;
-
 #ifndef min
 # define min(a,b)	(((a) < (b)) ? (a) : (b))
 # define max(a,b)	(((a) > (b)) ? (a) : (b))
@@ -363,14 +231,18 @@ typedef unsigned int rend_t;
  *****************************************************************************
  */
 
+#if defined (NO_OLD_SELECTION) && defined(NO_NEW_SELECTION)
+# error if you disable both selection styles, how can you select, silly?
+#endif
+
 /* sort out conflicts in feature.h */
 #ifdef KANJI
-# undef BIG5			/* remove Chinese big5 support        */
+# undef ZH			/* remove Chinese big5 support        */
 # undef GREEK_SUPPORT		/* Kanji/Greek together is too weird  */
 # undef DEFINE_XTERM_COLOR	/* since kterm-color doesn't exist?   */
 #endif
 
-#ifdef BIG5 
+#ifdef ZH 
 # undef KANJI			/* can't put Chinese/Kanji together   */
 # undef GREEK_SUPPORT		
 # undef DEFINE_XTERM_COLOR
@@ -386,6 +258,11 @@ typedef unsigned int rend_t;
 
 /* COLORTERM, TERM environment variables */
 #define COLORTERMENV	"rxvt"
+#ifdef XPM_BACKGROUND
+# define COLORTERMENVFULL COLORTERMENV "-xpm"
+#else
+# define COLORTERMENVFULL COLORTERMENV
+#endif
 #ifndef TERMENV
 # ifdef KANJI
 #  define TERMENV	"kterm"
@@ -404,66 +281,6 @@ typedef unsigned int rend_t;
 
 /* now look for other badly set stuff */
 
-#ifdef UTMP_SUPPORT
-# ifndef UTMP_FILENAME
-#  ifdef UTMP_FILE
-#   define UTMP_FILENAME		UTMP_FILE
-#  else
-#   ifdef _PATH_UTMP
-#    define UTMP_FILENAME		_PATH_UTMP
-#   else
-#    ifdef RXVT_UTMP_FILE
-#     define UTMP_FILENAME		RXVT_UTMP_FILE
-#    else
-#     error define UTMP_FILENAME in config.h
-#    endif
-#   endif
-#  endif
-# endif
-# ifdef USE_SYSV_UTMP
-#  ifndef USER_PROCESS
-#   define USER_PROCESS		7
-#  endif
-#  ifndef DEAD_PROCESS
-#   define DEAD_PROCESS		8
-#  endif
-# endif
-# ifdef HAVE_UTMPX_H
-#  define UTMP			struct utmpx
-#  define setutent		setutxent
-#  define getutent		getutxent
-#  define getutid		getutxid
-#  define endutent		endutxent
-#  define pututline		pututxline
-# else
-#  define UTMP			struct utmp
-# endif
-#endif
-
-#ifdef WTMP_SUPPORT
-# ifdef HAVE_UTMPX_H		/* means we're on Solaris (or Irix?) */
-#  undef WTMP_FILENAME
-#  define WTMP_FILENAME		WTMPX_FILE
-#  define update_wtmp		updwtmpx
-# else
-#  ifndef WTMP_FILENAME
-#   ifdef WTMP_FILE
-#    define WTMP_FILENAME		WTMP_FILE
-#   else
-#    ifdef _PATH_WTMP
-#     define WTMP_FILENAME		_PATH_WTMP
-#    else
-#     ifdef RXVT_WTMP_FILE
-#      define WTMP_FILENAME		RXVT_WTMP_FILE
-#     else
-#      error define WTMP_FILENAME in config.h
-#     endif
-#    endif
-#   endif
-#  endif
-# endif
-#endif
-
 #if !defined (EACCESS) && defined(EAGAIN)
 # define EACCESS EAGAIN
 #endif
@@ -479,8 +296,6 @@ typedef unsigned int rend_t;
 
 /* gap between text and window edges (could be configurable) */
 #define TermWin_internalBorder	2
-#define MAX_COLS		200	/* Maximum number of columns */
-#define MAX_ROWS		128	/* Maximum number of rows */
 
 /* width of scrollBar, menuBar shadow, must be 1 or 2 */
 #ifdef HALFSHADOW
@@ -495,7 +310,7 @@ typedef unsigned int rend_t;
 #else
 # if !defined (SB_WIDTH) || (SB_WIDTH < 8)
 #  undef SB_WIDTH
-#  define SB_WIDTH		10	/* scrollBar width */
+#  define SB_WIDTH		11	/* scrollBar width */
 # endif
 #endif				/* XTERM_SCROLLBAR */
 
@@ -542,27 +357,32 @@ enum {
     SECONDARY
 };
 
-#define RS_None			0		/* Normal */
-#define RS_Cursor		0x01000000u	/* cursor location */
-#define RS_Select		0x02000000u	/* selected text */
-#define RS_RVid			0x04000000u	/* reverse video */
-#define RS_Uline		0x08000000u	/* underline */
-#define RS_acsFont		0x10000000u	/* ACS graphics character set */
-#define RS_ukFont		0x20000000u	/* UK character set */
-#define RS_fontMask		(RS_acsFont|RS_ukFont)
-#if defined(KANJI) || defined(BIG5)
-#define RS_multi0		0x40000000u	/* only multibyte characters */
-#define RS_multi1		0x80000000u	/* multibyte 1st byte */
-#define RS_multi2		(RS_multi0|RS_multi1)	/* multibyte 2nd byte */
-#define RS_multiMask		(RS_multi0|RS_multi1)	/* multibyte mask */
-#endif
-#define RS_fgMask		0x00001F00u	/* 32 colors */
-#define RS_Bold			0x00008000u	/* bold */
-#define RS_bgMask		0x001F0000u	/* 32 colors */
-#define RS_Blink		0x00800000u	/* blink */
-#define RS_Dirty		0x00400000u	/* forced update of char */
+/* all basic bit-flags in first/lower 16 bits */
 
-#define RS_attrMask		(0xFF000000u|RS_Bold|RS_Blink)
+#define RS_None			0		/* Normal */
+#define RS_fgMask		0x0000001Fu	/* 32 colors */
+#define RS_bgMask		0x000003E0u	/* 32 colors */
+#define RS_Bold			0x00000400u	/* bold */
+#define RS_Blink		0x00000800u	/* blink */
+#define RS_RVid			0x00001000u	/* reverse video */
+#define RS_Uline		0x00002000u	/* underline */
+#define RS_acsFont		0x00004000u	/* ACS graphics char set */
+#define RS_ukFont		0x00008000u	/* UK character set */
+#define RS_fontMask		(RS_acsFont|RS_ukFont)
+#define RS_baseattrMask		(RS_Bold|RS_Blink|RS_RVid|RS_Uline)
+
+/* all other bit-flags in upper 16 bits */
+
+#ifdef MULTICHAR_SET
+# define RS_multi0		0x10000000u	/* only multibyte characters */
+# define RS_multi1		0x20000000u	/* multibyte 1st byte */
+# define RS_multi2		(RS_multi0|RS_multi1)	/* multibyte 2nd byte */
+# define RS_multiMask		(RS_multi0|RS_multi1)	/* multibyte mask */
+#else
+# define RS_multiMask		0
+#endif
+
+#define RS_attrMask		(RS_baseattrMask|RS_fontMask|RS_multiMask)
 
 #define	Opt_console		(1LU<<0)
 #define Opt_loginShell		(1LU<<1)
@@ -597,64 +417,131 @@ enum {
 #define XTerm_restoreFG		39	/* change default fg color */
 #define XTerm_restoreBG		49	/* change default bg color */
 
-#define NFONTS			5	/* number of fonts */
-
 #define restoreFG		39	/* restore default fg color */
 #define restoreBG		49	/* restore default bg color */
 
+/* Words starting with `Color_' are colours.  Others are counts */
+
 enum colour_list {
-    fgColor,
-    bgColor,
-    minColor,				/* 2 */
-    BlackColor = minColor,
-    Red3Color,
-    Green3Color,
-    Yellow3Color,
-    Blue3Color,
-    Magenta3Color,
-    Cyan3Color,
-    maxColor,				/* minColor + 7 */
+    Color_fg = 0,
+    Color_bg,
+    minCOLOR,				/* 2 */
+    Color_Black = minCOLOR,
+    Color_Red3,
+    Color_Green3,
+    Color_Yellow3,
+    Color_Blue3,
+    Color_Magenta3,
+    Color_Cyan3,
+    maxCOLOR,				/* minCOLOR + 7 */
 #ifndef NO_BRIGHTCOLOR
-    AntiqueWhiteColor = maxColor,
-    minBright,				/* maxColor + 1 */
-    Grey25Color = minBright,
-    RedColor,
-    GreenColor,
-    YellowColor,
-    BlueColor,
-    MagentaColor,
-    CyanColor,
-    maxBright,				/* minBright + 7 */
-    WhiteColor = maxBright,
+    Color_AntiqueWhite = maxCOLOR,
+    minBrightCOLOR,			/* maxCOLOR + 1 */
+    Color_Grey25 = minBrightCOLOR,
+    Color_Red,
+    Color_Green,
+    Color_Yellow,
+    Color_Blue,
+    Color_Magenta,
+    Color_Cyan,
+    maxBrightCOLOR,			/* minBrightCOLOR + 7 */
+    Color_White = maxBrightCOLOR,
 #else
-    WhiteColor = maxColor,
+    Color_White = maxCOLOR,
 #endif
 #ifndef NO_CURSORCOLOR
-    cursorColor,
-    cursorColor2,
+    Color_cursor,
+    Color_cursor2,
 #endif
-    pointerColor,
-    borderColor,
+    Color_pointer,
+    Color_border,
 #ifndef NO_BOLDUNDERLINE
-    colorBD,
-    colorUL,
+    Color_BD,
+    Color_UL,
 #endif
 #ifdef KEEP_SCROLLCOLOR
-    scrollColor,
+    Color_scroll,
+    Color_trough,
 #endif
     NRS_COLORS,				/* */
 #ifdef KEEP_SCROLLCOLOR
-    topShadowColor = NRS_COLORS,
-    bottomShadowColor,
-    TOTAL_COLORS			/* */
+    Color_topShadow = NRS_COLORS,
+    Color_bottomShadow,
+    TOTAL_COLORS			/* upto 28 */
 #else
     TOTAL_COLORS = NRS_COLORS		/* */
 #endif
 };
 
-#define NSHADOWCOLORS		(TOTAL_COLORS - NRS_COLORS)
+#define DEFAULT_RSTYLE		(RS_None | (Color_fg) | (Color_bg<<5))
 
-#define DEFAULT_RSTYLE		(RS_None | (fgColor<<8) | (bgColor<<16))
+/*
+ * This resource list should match xdefaults.c
+ * - though not necessarily in order
+ */
+enum Rs_resource_list {
+    Rs_display_name = 0, 
+    Rs_term_name,
+    Rs_geometry,
+    Rs_reverseVideo,
+    Rs_color,
+    Rs_font = Rs_color + NRS_COLORS,
+    Rs_iconName = Rs_font + NFONTS,
+#ifdef MULTICHAR_SET
+    Rs_mfont,
+    Rs_multichar_encoding = Rs_mfont + NFONTS,
+#endif
+    Rs_name,
+    Rs_title,
+#if defined (XPM_BACKGROUND) || (MENUBAR_MAX)
+    Rs_path,
+#endif
+#ifdef XPM_BACKGROUND
+    Rs_backgroundPixmap,
+#endif
+#if (MENUBAR_MAX)
+    Rs_menu,
+#endif
+#ifndef NO_BOLDFONT
+    Rs_boldFont,
+#endif
+#ifdef GREEK_SUPPORT
+    Rs_greek_keyboard,
+#endif
+    Rs_loginShell,
+    Rs_scrollBar,
+    Rs_scrollBar_right,
+    Rs_scrollBar_floating,
+    Rs_scrollTtyOutputInh,
+    Rs_scrollKeypress,
+    Rs_saveLines,
+    Rs_utmpInhibit,
+    Rs_visualBell,
+#if ! defined(NO_MAPALERT) && defined(MAPALERT_OPTION)
+    Rs_mapAlert,
+#endif
+#ifdef META8_OPTION
+    Rs_meta8,
+#endif
+#ifndef NO_BACKSPACE_KEY
+    Rs_backspace_key,
+#endif
+#ifndef NO_DELETE_KEY
+    Rs_delete_key,
+#endif
+    Rs_selectstyle,
+#ifdef PRINTPIPE
+    Rs_print_pipe,
+#endif
+#if defined (HOTKEY_CTRL) || defined (HOTKEY_META)
+    Rs_bigfont_key,
+    Rs_smallfont_key,
+#endif
+#ifdef CUTCHAR_RESOURCE
+    Rs_cutchars,
+#endif
+    TOTAL_RS
+} ;
 
 /*
  * number of graphics points
@@ -668,13 +555,19 @@ enum colour_list {
  * MACRO DEFINES
  *****************************************************************************
  */
-#ifndef DEBUG_MALLOC
-# define MALLOC(sz)		malloc (sz)
-# define CALLOC(type, n)	calloc ((n),sizeof(type))
-# define REMALLOC(mem,sz)	((mem) ? realloc ((mem),(sz)) : malloc(sz))
-# define FREE(ptr)		free (ptr)
+#define STRCPY(a, b)		strcpy ((char *)(a), (const char *) (b))
+#define STRNCPY(a, b, c)	strncpy ((char *)(a), (const char *) (b), (c))
+#ifdef NO_RMEMSET
+# define MEMSET(a, b, c)	memset ((a), (b), (c))
+#else
+# define MEMSET(a, b, c)	rmemset ((a), (b), (R_int_p_t) (c))
 #endif
 
+#define MALLOC(sz)		malloc (sz)
+#define CALLOC(type, n)		calloc ((n), sizeof(type))
+#define REALLOC(mem, sz)	((mem) ? realloc ((mem), (sz)) : malloc(sz))
+#define FREE(ptr)		free (ptr)
+
 /* convert pixel dimensions to row/column values */
 #define Pixel2Col(x)		Pixel2Width((x) - TermWin_internalBorder)
 #define Pixel2Row(y)		Pixel2Height((y) - TermWin_internalBorder)
@@ -689,17 +582,15 @@ enum colour_list {
 #define TermWin_TotalHeight()	(TermWin.height + 2 * TermWin_internalBorder)
 
 #define Xscreen			DefaultScreen(Xdisplay)
-
-
 #define Xroot			DefaultRootWindow(Xdisplay)
 
 /* how to build & extract colors and attributes */
-#define GET_FGCOLOR(r)		(((r) & RS_fgMask)>>8)
-#define GET_BGCOLOR(r)		(((r) & RS_bgMask)>>16)
+#define GET_FGCOLOR(r)		(((r) & RS_fgMask))
+#define GET_BGCOLOR(r)		(((r) & RS_bgMask)>>5)
 #define GET_ATTR(r)		(((r) & RS_attrMask))
 #define GET_BGATTR(r)		(((r) & (RS_attrMask | RS_bgMask)))
-#define SET_FGCOLOR(r,fg)	(((r) & ~RS_fgMask)  | ((fg)<<8))
-#define SET_BGCOLOR(r,bg)	(((r) & ~RS_bgMask)  | ((bg)<<16))
+#define SET_FGCOLOR(r,fg)	(((r) & ~RS_fgMask)  | (fg))
+#define SET_BGCOLOR(r,bg)	(((r) & ~RS_bgMask)  | ((bg)<<5))
 #define SET_ATTR(r,a)		(((r) & ~RS_attrMask)| (a))
 
 #define scrollbar_visible()	(scrollBar.state)
@@ -714,8 +605,15 @@ enum colour_list {
 #define scrollbar_setUp()	scrollBar.state = 'U'
 #define scrollbar_setDn()	scrollBar.state = 'D'
 
-#define scrollbar_upButton(y)	((y) < scrollBar.beg)
-#define scrollbar_dnButton(y)	((y) > scrollBar.end)
+#ifdef NEXT_SCROLLBAR
+# define scrollbar_dnval()	(scrollBar.end + (SB_WIDTH + 1))
+# define scrollbar_upButton(y)	((y) > scrollBar.end \
+				 && (y) <= scrollbar_dnval())
+# define scrollbar_dnButton(y)	((y) > scrollbar_dnval())
+#else
+# define scrollbar_upButton(y)	((y) < scrollBar.beg)
+# define scrollbar_dnButton(y)	((y) > scrollBar.end)
+#endif
 
 #define scrollbar_above_slider(y)	((y) < scrollBar.top)
 #define scrollbar_below_slider(y)	((y) > scrollBar.bot)
@@ -729,32 +627,20 @@ enum colour_list {
 # define menuBar_TotalHeight()	(menuBar_height() + SHADOW + menuBar_margin)
 # define isMenuBarWindow(w)	((w) == menuBar.win)
 #else
-# define menubar_dispatch(str)	((void)0)
-# define menubar_read(filename)	((void)0)
-# define menubar_expose()	((void)0)
-# define menubar_control(ev)	((void)0)
-# define menubar_mapping(map)	(0)
-# define menubar_visible()	(0)
 # define isMenuBarWindow(w)	(0)
+# define menuBar_height()	(0)
+# define menuBar_TotalHeight()	(0)
+# define menubar_visible()	(0)
 #endif
 
-#ifdef RXVT_GRAPHICS
-# define Gr_ButtonPress(x,y)	Gr_ButtonReport ('P',(x),(y))
-# define Gr_ButtonRelease(x,y)	Gr_ButtonReport ('R',(x),(y))
-# define GR_DISPLAY(x)		if (Gr_Displayed()) (x)
-# define GR_NO_DISPLAY(x)	if (!Gr_Displayed()) (x)
+#ifdef XPM_BACKGROUND
+# define XPMClearArea(a, b, c, d, e, f, g)	XClearArea((a), (b), (c), (d), (e), (f), (g))
 #else
-# define Gr_ButtonPress(x,y)	((void)0)
-# define Gr_ButtonRelease(x,y)	((void)0)
-# define Gr_scroll(count)	((void)0)
-# define Gr_ClearScreen()	((void)0)
-# define Gr_ChangeScreen()	((void)0)
-# define Gr_expose(win)		((void)0)
-# define Gr_Resize(w,h)		((void)0)
-# define Gr_reset()		((void)0)
-# define GR_DISPLAY(x)
-# define GR_NO_DISPLAY(x)	(x)
+# define XPMClearArea(a, b, c, d, e, f, g)
 #endif
+
+#define Gr_ButtonPress(x,y)	Gr_ButtonReport ('P',(x),(y))
+#define Gr_ButtonRelease(x,y)	Gr_ButtonReport ('R',(x),(y))
 /*
  *****************************************************************************
  * VARIABLES
@@ -769,7 +655,7 @@ enum colour_list {
 #ifdef PREFER_24BIT
 EXTERN Colormap     Xcmap;
 EXTERN int          Xdepth;
-EXTERN Visual    *  Xvisual;
+EXTERN Visual      *Xvisual;
 #else
 # define Xcmap			DefaultColormap(Xdisplay,Xscreen)
 # define Xdepth			DefaultDepth(Xdisplay,Xscreen)
@@ -780,70 +666,44 @@ EXTERN Visual    *  Xvisual;
 # endif
 #endif
 
-EXTERN TermWin_t TermWin;
-EXTERN scrollBar_t scrollBar;
-EXTERN menuBar_t menuBar;
-EXTERN Display *Xdisplay;
-EXTERN unsigned long Options;
-EXTERN int	sb_shadow;
-EXTERN int	delay_menu_drawing;
-EXTERN const char *display_name;
-EXTERN const char *rs_name;	/* client instance (resource name) */
-EXTERN const char *rs_term_name;
-EXTERN const char *rs_color[NRS_COLORS];
-EXTERN Pixel    PixColors[NRS_COLORS + NSHADOWCOLORS];
-EXTERN const char *rs_font[NFONTS];
-EXTERN const char *rs_title;	/* title name for window */
-EXTERN const char *rs_iconName;	/* icon name for window */
-EXTERN const char *rs_geometry;	/* window geometry */
-EXTERN const char *rs_saveLines;	/* scrollback buffer [lines] */
-EXTERN const char *rs_cutchars;	/* chars for selection boundaries */
-
-#ifdef KANJI
-EXTERN const char *rs_kfont[NFONTS];
+EXTERN TermWin_t	TermWin;
+EXTERN scrollBar_t	scrollBar;
+EXTERN menuBar_t	menuBar;
+EXTERN Display	       *Xdisplay;
+EXTERN unsigned long	Options;
+EXTERN XSizeHints       szHint;
+EXTERN int		sb_shadow;
+EXTERN Pixel		PixColors[TOTAL_COLORS];
+#ifdef INEXPENSIVE_LOCAL_X_CALLS
+EXTERN int              display_is_local;
 #endif
 
-#ifdef BIG5
-EXTERN const char *rs_cfont[NFONTS];
-#endif
+EXTERN const char      *rs[TOTAL_RS];
 
-#ifndef NO_BOLDFONT
-EXTERN const char *rs_boldFont;
+#ifndef NO_BACKSPACE_KEY
+EXTERN const char      *key_backspace;
 #endif
-
-#ifdef PRINTPIPE
-EXTERN const char *rs_print_pipe;
+#ifndef NO_DELETE_KEY
+EXTERN const char      *key_delete;
 #endif
-
 #ifndef NO_BRIGHTCOLOR
-EXTERN unsigned int colorfgbg;
+EXTERN unsigned int	colorfgbg;
 #endif
-
 #ifdef KEYSYM_RESOURCE
 EXTERN const unsigned char *KeySym_map[256];
 #endif
-
 #if defined (HOTKEY_CTRL) || defined (HOTKEY_META)
-EXTERN KeySym   ks_bigfont;
-EXTERN KeySym   ks_smallfont;
+EXTERN KeySym		ks_bigfont;
+EXTERN KeySym		ks_smallfont;
 #endif
 
-#if (MENUBAR_MAX)
-EXTERN const char *rs_menu;
-#endif
-
-#if defined (XPM_BACKGROUND) || (MENUBAR_MAX)
-EXTERN const char *rs_path;
-# ifdef XPM_BACKGROUND
-EXTERN const char *rs_backgroundPixmap;
-# endif				/* XPM_BACKGROUND */
-#endif				/* defined (XPM_BACKGROUND) || (MENUBAR_MAX) */
-
 /*
  *****************************************************************************
  * PROTOTYPES
  *****************************************************************************
  */
+#include "screen.h"
+
 #ifdef PROTOTYPES
 # define __PROTO(p)	p
 #else
