@@ -1,7 +1,9 @@
 /*--------------------------------*-C-*---------------------------------*
  * File:	scrollbar.c
  *----------------------------------------------------------------------*
- * Copyright (C) 1997 1998 mj olesen <olesen@me.QueensU.CA>
+ * $Id: scrollbar.c,v 1.15 1998/11/16 19:55:54 mason Exp $
+ *
+ * Copyright (C) 1997,1998 mj olesen <olesen@me.QueensU.CA>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,10 +18,10 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
  *----------------------------------------------------------------------*/
 
 #include "rxvt.h"		/* NECESSARY */
+#include "scrollbar.intpro"	/* PROTOS for internal routines */
 
 /*----------------------------------------------------------------------*
  */
@@ -38,10 +40,10 @@ Error, check scrollbar width (SB_WIDTH). It must be 15 for XTERM_SCROLLBAR
 #else				/* XTERM_SCROLLBAR */
 static GC       topShadowGC, botShadowGC;
 
-/* draw triangular up button with a shadow of SHADOW (1 or 2) pixels */
-/* PROTO */
+/* draw triangular button with a shadow of SHADOW (1 or 2) pixels */
+/* INTPROTO */
 void
-Draw_up_button(int x, int y, int state)
+Draw_button(int x, int y, int state, int dirn)
 {
     const unsigned int sz = (SB_WIDTH), sz2 = (SB_WIDTH / 2);
     XPoint          pt[3];
@@ -56,124 +58,72 @@ Draw_up_button(int x, int y, int state)
 	top = botShadowGC;
 	bot = topShadowGC;
 	break;
-    case 0:
+    default:
 	top = bot = scrollbarGC;
 	break;
     }
 
 /* fill triangle */
     pt[0].x = x;
-    pt[0].y = y + sz - 1;
     pt[1].x = x + sz - 1;
-    pt[1].y = y + sz - 1;
     pt[2].x = x + sz2;
-    pt[2].y = y;
-    XFillPolygon(Xdisplay, scrollBar.win, scrollbarGC,
-		 pt, 3, Convex, CoordModeOrigin);
-
-/* draw base */
-    XDrawLine(Xdisplay, scrollBar.win, bot,
-	      pt[0].x, pt[0].y, pt[1].x, pt[1].y);
-
-/* draw shadow */
-    pt[1].x = x + sz2 - 1;
-    pt[1].y = y;
-    XDrawLine(Xdisplay, scrollBar.win, top,
-	      pt[0].x, pt[0].y, pt[1].x, pt[1].y);
-#if (SHADOW > 1)
-/* doubled */
-    pt[0].x++;
-    pt[0].y--;
-    pt[1].y++;
-    XDrawLine(Xdisplay, scrollBar.win, top,
-	      pt[0].x, pt[0].y, pt[1].x, pt[1].y);
-#endif
-/* draw shadow */
-    pt[0].x = x + sz2;
-    pt[0].y = y;
-    pt[1].x = x + sz - 1;
-    pt[1].y = y + sz - 1;
-    XDrawLine(Xdisplay, scrollBar.win, bot,
-	      pt[0].x, pt[0].y, pt[1].x, pt[1].y);
-#if (SHADOW > 1)
-/* doubled */
-    pt[0].y++;
-    pt[1].x--;
-    pt[1].y--;
-    XDrawLine(Xdisplay, scrollBar.win, bot,
-	      pt[0].x, pt[0].y, pt[1].x, pt[1].y);
-#endif
-}
-
-/* draw triangular down button with a shadow of SHADOW (1 or 2) pixels */
-/* PROTO */
-void
-Draw_dn_button(int x, int y, int state)
-{
-    const unsigned int sz = (SB_WIDTH), sz2 = (SB_WIDTH / 2);
-    XPoint          pt[3];
-    GC              top, bot;
-
-    switch (state) {
-    case +1:
-	top = topShadowGC;
-	bot = botShadowGC;
-	break;
-    case -1:
-	top = botShadowGC;
-	bot = topShadowGC;
-	break;
-    case 0:
-	top = bot = scrollbarGC;
-	break;
+    if (dirn == UP) {
+	pt[0].y = pt[1].y = y + sz - 1;
+	pt[2].y = y;
+    } else {
+	pt[0].y = pt[1].y = y;
+	pt[2].y = y + sz - 1;
     }
-
-/* fill triangle */
-    pt[0].x = x;
-    pt[0].y = y;
-    pt[1].x = x + sz - 1;
-    pt[1].y = y;
-    pt[2].x = x + sz2;
-    pt[2].y = y + sz;
     XFillPolygon(Xdisplay, scrollBar.win, scrollbarGC,
 		 pt, 3, Convex, CoordModeOrigin);
 
 /* draw base */
+    XDrawLine(Xdisplay, scrollBar.win, (dirn == UP ? bot : top),
+	      pt[0].x, pt[0].y, pt[1].x, pt[1].y);
+
+/* draw shadow on left */
+    pt[1].x = x + sz2 - 1;
+    pt[1].y = y + (dirn == UP ? 0 : sz - 1);
     XDrawLine(Xdisplay, scrollBar.win, top,
 	      pt[0].x, pt[0].y, pt[1].x, pt[1].y);
 
-/* draw shadow */
-    pt[1].x = x + sz2 - 1;
-    pt[1].y = y + sz - 1;
-    XDrawLine(Xdisplay, scrollBar.win, top,
-	      pt[0].x, pt[0].y, pt[1].x, pt[1].y);
 #if (SHADOW > 1)
 /* doubled */
     pt[0].x++;
-    pt[0].y++;
-    pt[1].y--;
+    if (dirn == UP) {
+	pt[0].y--;
+	pt[1].y++;
+    } else {
+	pt[0].y++;
+	pt[1].y--;
+    }
     XDrawLine(Xdisplay, scrollBar.win, top,
 	      pt[0].x, pt[0].y, pt[1].x, pt[1].y);
 #endif
-/* draw shadow */
-    pt[0].x = x + sz2;
-    pt[0].y = y + sz - 1;
+/* draw shadow on right */
     pt[1].x = x + sz - 1;
-    pt[1].y = y;
+    /* pt[2].x = x + sz2; */
+    pt[1].y = y + (dirn == UP ? sz - 1 : 0);
+    pt[2].y = y + (dirn == UP ? 0 : sz - 1);
     XDrawLine(Xdisplay, scrollBar.win, bot,
-	      pt[0].x, pt[0].y, pt[1].x, pt[1].y);
+	      pt[2].x, pt[2].y, pt[1].x, pt[1].y);
 #if (SHADOW > 1)
 /* doubled */
-    pt[0].y--;
     pt[1].x--;
-    pt[1].y++;
+    if (dirn == UP) {
+	pt[2].y++;
+	pt[1].y--;
+    } else {
+	pt[2].y--;
+	pt[1].y++;
+    }
     XDrawLine(Xdisplay, scrollBar.win, bot,
-	      pt[0].x, pt[0].y, pt[1].x, pt[1].y);
+	      pt[2].x, pt[2].y, pt[1].x, pt[1].y);
 #endif
 }
-#endif				/* XTERM_SCROLLBAR */
+#endif				/* ! XTERM_SCROLLBAR */
 
-/* PROTO */
+/* EXTPROTO */
 int
 scrollbar_mapping(int map)
 {
@@ -181,6 +131,8 @@ scrollbar_mapping(int map)
 
     if (map && !scrollbar_visible()) {
 	scrollBar.state = 1;
+	if (scrollBar.win == 0)
+	    return 0;
 	XMapWindow(Xdisplay, scrollBar.win);
 	change = 1;
     } else if (!map && scrollbar_visible()) {
@@ -191,7 +143,7 @@ scrollbar_mapping(int map)
     return change;
 }
 
-/* PROTO */
+/* EXTPROTO */
 int
 scrollbar_show(int update)
 {
@@ -213,33 +165,35 @@ scrollbar_show(int update)
 	    exit(EXIT_FAILURE);
 	}
 	gcvalue.fill_style = FillOpaqueStippled;
-	gcvalue.foreground = PixColors[fgColor];
-	gcvalue.background = PixColors[bgColor];
+	gcvalue.foreground = PixColors[Color_fg];
+	gcvalue.background = PixColors[Color_bg];
 
 	scrollbarGC = XCreateGC(Xdisplay, scrollBar.win,
 				GCForeground | GCBackground |
 				GCFillStyle | GCStipple,
 				&gcvalue);
-        gcvalue.foreground = PixColors[borderColor];
+        gcvalue.foreground = PixColors[Color_border];
 	ShadowGC = XCreateGC(Xdisplay, scrollBar.win, GCForeground, &gcvalue);
 #else				/* XTERM_SCROLLBAR */
 	sb_width = SB_WIDTH;
-	gcvalue.foreground = (Xdepth <= 2 ? PixColors[fgColor]
-					  : PixColors[scrollColor]);
-	scrollbarGC = XCreateGC(Xdisplay, scrollBar.win, GCForeground,
-				&gcvalue);
 
+	gcvalue.foreground = PixColors[Color_trough];
 	if (sb_shadow) {
 	    XSetWindowBackground(Xdisplay, scrollBar.win, gcvalue.foreground);
 	    XClearWindow(Xdisplay, scrollBar.win);
 	}
 
-	gcvalue.foreground = PixColors[topShadowColor];
+	gcvalue.foreground = (Xdepth <= 2 ? PixColors[Color_fg]
+					  : PixColors[Color_scroll]);
+	scrollbarGC = XCreateGC(Xdisplay, scrollBar.win, GCForeground,
+				&gcvalue);
+
+	gcvalue.foreground = PixColors[Color_topShadow];
 	topShadowGC = XCreateGC(Xdisplay, scrollBar.win,
 				GCForeground,
 				&gcvalue);
 
-	gcvalue.foreground = PixColors[bottomShadowColor];
+	gcvalue.foreground = PixColors[Color_bottomShadow];
 	botShadowGC = XCreateGC(Xdisplay, scrollBar.win,
 				GCForeground,
 				&gcvalue);
@@ -285,10 +239,10 @@ scrollbar_show(int update)
     XDrawLine(Xdisplay, scrollBar.win, ShadowGC,
 	      xsb ? 0 : 14, scrollBar.beg, xsb ? 0 : 14, scrollBar.end);
 #else
-#ifdef SB_BORDER
+# ifdef SB_BORDER
    XDrawLine(Xdisplay, scrollBar.win, botShadowGC,
-        SB_WIDTH, 0, SB_WIDTH, scrollBar.end + SB_WIDTH) ;
-#endif
+	     SB_WIDTH, 0, SB_WIDTH, scrollBar.end + SB_WIDTH) ;
+# endif
    XFillRectangle(Xdisplay, scrollBar.win, scrollbarGC,
 		   sb_shadow, scrollBar.top,
 		   sb_width, (scrollBar.bot - scrollBar.top));
@@ -309,10 +263,30 @@ scrollbar_show(int update)
 /*
  * Redraw scrollbar arrows
  */
-    Draw_up_button(sb_shadow, sb_shadow, (scrollbar_isUp()? -1 : +1));
-    Draw_dn_button(sb_shadow, (scrollBar.end + 1), (scrollbar_isDn()? -1 : +1));
+# ifdef NEXT_SCROLLBAR
+    Draw_button(sb_shadow, (scrollBar.end + 1),
+		(scrollbar_isUp() ? -1 : +1), UP);
+    Draw_button(sb_shadow, (scrollBar.end + SB_WIDTH + 2),
+		(scrollbar_isDn() ? -1 : +1), DN);
+# else
+    Draw_button(sb_shadow, sb_shadow,
+		(scrollbar_isUp() ? -1 : +1), UP);
+    Draw_button(sb_shadow, (scrollBar.end + 1),
+		(scrollbar_isDn() ? -1 : +1), DN);
+# endif
 #endif				/* XTERM_SCROLLBAR */
 
     return 1;
 }
+
+/* EXTPROTO */
+void
+map_scrollBar(int map)
+{
+    if (scrollbar_mapping(map)) {
+	resize_all_windows();
+	scr_touch();
+    }
+}
+
 /*----------------------- end-of-file (C source) -----------------------*/
